(*
 Main file for program execution
 
 This adjusts the interpreter according to the command line arguments
 and loads the given files into the interpreter.
 
 author: Christian Mueller
*)

local 
    open AbstractSyntax Environment Toplevel PrettyPrinter ParserSupport
in

(* evaluates the command line arguments *)
    
fun evalModes ("-lw"::s::xr) (mode,lw,sub) =  
    let 
	val n =  valOf (Int.fromString s) 
	    handle Option => raise Fail "-lw: number expected"
    in  
	PrettyPrinter.setLineWidth n;
	if lw then raise Fail "line mode set twice" 
	else evalModes xr (mode,true,sub)
    end 

  | evalModes ("-s"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: simply typed\n";
	  setMode S; 
	  evalModes xr (true,lw,sub))
	
  | evalModes ("-f"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: System F\n";
	  setMode F; 
	  evalModes xr (true,lw,sub))
	
  | evalModes ("-fw"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: System Fw\n";
	  setMode Fw; 
	  evalModes xr (true,lw,sub))
	
  | evalModes ("-cc"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: CC\n";
	  setMode CC; 
	  evalModes xr (true,lw,sub))
	
  | evalModes ("-r"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: type reconstruction\n";
	  setMode R; 						
	  evalModes xr (true,lw,sub))

  | evalModes ("-rg"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: type reconstruction, graph based\n";
	  setMode RG; 						
	  evalModes xr (true,lw,sub))

  | evalModes ("-ud"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: user defined\n";
	  setMode UD; 						
	  evalModes xr (true,lw,sub))

  | evalModes ("-u"::xr) (mode,lw,sub)  = 
    if mode then raise Fail "you can only give one mode selector"
    else (print "--> mode: untyped\n";
	  setMode U; 						
	  evalModes xr (true,lw,sub))

  | evalModes ("-sub"::xr) (mode,lw,sub)  = 
    if sub then raise Fail "please switch to sub only once"
    else (print "--> subtyping: enabled \n";
	  setSub true; 						
	  evalModes xr (mode,lw,true))

  | evalModes xs (mode,_,_) = (mode,xs)


(* sets the interpreter mode according to the command line arguments *)

fun evalCLArgs env xs = 
    let
	val (mode_set,xr) = evalModes xs (false,false,false)
    in
	if mode_set then () else (print "--> mode: simply typed\n"; setMode S);
	print "\n";
	(executeCommandList env (parse_files xr),Parser)
    end 
		   				  
fun printlst env xs =
    let
	fun pr nil = ()
	  | pr [e] = pplc env e
	  | pr (e::er) = (pplc env e; pprint ", "; pr er)
    in
	pr xs;
	PrettyPrinter.ppflush ()
    end

(* start the interpreter *)

val _ = print ("\n--> Cube Interpreter\n--> type \"help;\" and press RETURN"
	       ^" to get infos about available commands\n")

val _ = 
    let             
	val env = Environment.create()
	val (env',parser) = evalCLArgs env (CommandLine.arguments ())      
	    handle exn => (errorhandler (Environment.create()) exn; 
			   raise Lexer.Exit)
    in 
	Info.setCurrentFile "Toplevel input";
	run env' parser
    end
	handle Fail s => print ("\nerror: "^s^"\n\n")
	     | Lexer.Exit => (PrettyPrinter.ppflush (); 
			      print ("<-- CU \n"))    

end
