(*
 The user can implement its own type-checker into this file.
 Look at the signature for the interface format.

 author: Christian Mueller
*)


local
    open AbstractSyntax LCSupport Support
in

(*
 Use these exception if you want to throw an error. 
 - UserError simply prints out the given string
 - UserErrorWinfo prints out a mixture of strings and pretty-printed 
   expressions 
*)

exception UserError of string

exception UserErrorWInfo of info * (mixture list)

(*

 You can implement your own evaluator here. Look at "AbstractSyntax.sml" for
 implementation details of the internal syntax.

 Suppose you only want to implement "check", then you can use eval from 
 LambdaCube:

    fun eval e = LambdaCube.eval e

 If you want to implement the evaluator for a restricted calculus, I suggest
 the following format:

    fun eval (Int i) = Int i
      | eval (Plus(i,e1,e2)) = 
	(case (eval e1, eval e2) of 
	     (Int (_,x),Int (_,y)) => Int(NONE,x+y) 
	   | (e1',e2')     => raise UserErrorWInfo (i,
						    [Str "evaluation of ",
						    E e1, Str "+",E e2,
						    Str " failed"]))
	     
	   | eval e = raise UserErrorWInfo (i,
					    [Str "\"eval\" is not yet ",
					     Str "implemented for expression ",
					     E e])

*)

fun eval _ = raise UserError "\"eval\" is not yet implemented"


(*

 You can implement your own type checker here. Look at "AbstractSyntax.sml" 
 for implementation details of the internal syntax.

 Suppose you only want to implement "eval", then you can use eval from 
 LambdaCube:

    fun check e = LambdaCube.check e

 or leave out type checking with 
 
    fun check e = Star NONE

 If you want to implement the type checker for a restricted calculus, I 
 suggest the following format:

    fun check (Int _) = IntType NONE
      | check (Plus(i,e1,e2)) = 
	(case (check e1, check e2) of 
	     (IntType _,IntType _) => IntType NONE
	   | (e1',e2') => raise UserErrorWInfo(i, 
				    [Str "wrongly typed +, ", 
				     Str "expected to Int arguments, but got",
				     E e1', Str " and ", E e2']))
      | check e = raise UserErrorWInfo(i,
				       [Str "\"check\" is not yet implemented",
					Str " for expression ",E e])

 If you need an environment, e.g. for type checking of abstractions,
 use the following interface:

 fun get env x s = raise UserError ("\"get\" is not yet implemented")

 fun check' env e =  raise UserError ("\"check'\" is not yet implemented")
     
 fun check e = check' nil e

 Look at the LambdaCube file for further information.

*)


fun check _ = raise UserError "\"check\" is not yet implemented"


end
